<?php

namespace Drupal\kinescope\Form;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\kinescope\Plugin\media\Source\Kinescope;
use Drupal\media_library\Form\AddFormBase;
use Drupal\media_library\MediaLibraryUiBuilder;
use Drupal\media_library\OpenerResolverInterface;
use Drupal\media\MediaTypeInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Creates a form to create media entities from Kinescope URLs.
 *
 * @internal
 *   Form classes are internal.
 */
class KinescopeForm extends AddFormBase {

  /**
   * Kinescope source.
   */
  protected Kinescope $kinescope;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    MediaLibraryUiBuilder $library_ui_builder,
    OpenerResolverInterface $opener_resolver,
    Kinescope $kinescope,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->libraryUiBuilder = $library_ui_builder;
    $this->viewBuilder = $this->entityTypeManager->getViewBuilder('media');
    $this->openerResolver = $opener_resolver;
    $this->kinescope = $kinescope;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('media_library.ui_builder'),
      $container->get('media_library.opener_resolver'),
      $container->get('plugin.manager.media.source')->createInstance('kinescope_media')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return parent::getBaseFormId() . '_kinescope';
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return [
      'validateUrl',
      'addButtonSubmit',
      'updateFormCallback',
      'preRenderAddedMedia',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return $this->getBaseFormId();
  }

  /**
   * {@inheritdoc}
   */
  protected function getMediaType(FormStateInterface $form_state) {
    if ($this->mediaType) {
      return $this->mediaType;
    }

    $media_type = parent::getMediaType($form_state);
    if (!$media_type->getSource() instanceof Kinescope) {
      throw new \InvalidArgumentException('Can only add media types which use an Kinescope source plugin.');
    }
    return $media_type;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildInputElement(array $form, FormStateInterface $form_state) {
    // Add a container to group the input elements for styling purposes.
    $form['container'] = [
      '#type' => 'container',
    ];

    $form['container']['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Add Kinescope via URL'),
      '#required' => TRUE,
      '#attributes' => [
        'placeholder' => 'https://kinescope.io/video-id',
      ],
    ];

    $form['container']['submit_add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add'),
      '#button_type' => 'primary',
      '#validate' => ['::validateUrl'],
      '#submit' => ['::addButtonSubmit'],
      '#ajax' => [
        'callback' => '::updateFormCallback',
        'wrapper' => 'media-library-wrapper',
        'url' => Url::fromRoute('media_library.ui'),
        'options' => [
          'query' => $this->getMediaLibraryState($form_state)->all() + [
            FormBuilderInterface::AJAX_FORM_REQUEST => TRUE,
          ],
        ],
      ],
    ];
    return $form;
  }

  /**
   * Validates the URL.
   *
   * @param array $form
   *   The complete form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function validateUrl(array &$form, FormStateInterface $form_state) {
    $url = $form_state->getValue('url');
    if (!Kinescope::validateUrl($url)) {
      $form_state->setErrorByName('url', $this->t('The provided URL is not a valid Kinescope URL.'));
    }
  }

  /**
   * Submit handler for the add button.
   *
   * @param array $form
   *   The form render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function addButtonSubmit(array $form, FormStateInterface $form_state) {
    $this->processInputValues([$form_state->getValue('url')], $form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function createMediaFromValue(MediaTypeInterface $media_type, EntityStorageInterface $media_storage, $source_field_name, $source_field_value) {
    /** @var \Drupal\media\Entity\Media $media */
    $media = $media_storage->create([
      'bundle' => $media_type->id(),
      $source_field_name => $source_field_value,
    ]);
    $media->setName($media->getName());

    $config = \Drupal::config(KinescopeSettingsForm::SETTINGS)->get('kinescope') ?? [];
    $api_token = $config['api_token'] ?? '';

    $video_id = Kinescope::getVideoId($source_field_value) ?? '';

    if (empty($video_id) || empty($api_token)) {
      return $media;
    }

    $client = new Client();
    try {
      $response = $client->get('https://api.kinescope.io/v1/videos/' . $video_id, [
        'headers' => [
          'Accept' => 'application/json',
          'Authorization' => 'Bearer ' . $api_token,
        ],
      ]);
      $json_data = json_decode($response->getBody(), TRUE);
      $name = $json_data['data']['title'] ?? '';
      if ($name) {
        $name = strip_tags($name);
        $media->setName($name);
      }
    }
    catch (RequestException $e) {
    }

    return $media;
  }

}
